/*
 * Copyright Elasticsearch B.V. and/or licensed to Elasticsearch B.V. under one
 * or more contributor license agreements. Licensed under the Elastic License
 * 2.0; you may not use this file except in compliance with the Elastic License
 * 2.0.
 */

import expect from '@kbn/expect';
import { getLifecycleMethods } from '../_get_lifecycle_methods';

export default function ({ getService, getPageObjects }) {
  const clusterList = getService('monitoringClusterList');
  const clusterOverview = getService('monitoringClusterOverview');
  const testSubjects = getService('testSubjects');
  const PageObjects = getPageObjects(['monitoring', 'header', 'common']);
  const alertsService = getService('monitoringAlerts');
  const browser = getService('browser');

  describe('Cluster listing', () => {
    describe('with trial license clusters', () => {
      const { setup, tearDown } = getLifecycleMethods(getService, getPageObjects);

      const UNSUPPORTED_CLUSTER_UUID = '6d-9tDFTRe-qT5GoBytdlQ';

      before(async () => {
        await setup('x-pack/test/functional/es_archives/monitoring/multicluster', {
          from: 'Aug 15, 2017 @ 21:00:00.000',
          to: 'Aug 16, 2017 @ 00:00:00.000',
        });

        await clusterList.closeAlertsModal();

        await clusterList.assertDefaults();
      });

      after(async () => {
        await tearDown();
      });

      afterEach(async () => {
        await clusterList.clearFilter();
      });

      describe('cluster table and toolbar', () => {
        it('shows 3 clusters sorted by name', async () => {
          const rows = await clusterList.getRows();
          expect(rows.length).to.be(3);
        });

        it('filters for a single cluster', async () => {
          await clusterList.setFilter('clusterone');
          const rows = await clusterList.getRows();
          expect(rows.length).to.be(1);
        });

        it('filters for non-existent cluster', async () => {
          await clusterList.setFilter('foobar');
          await clusterList.assertNoData();
        });
      });

      describe('cluster row actions', () => {
        it('clicking the basic cluster shows a toast message', async () => {
          const basicClusterLink = await clusterList.getClusterLink(UNSUPPORTED_CLUSTER_UUID);
          await basicClusterLink.click();
          expect(await testSubjects.exists('monitoringLicenseWarning', { timeout: 2000 })).to.be(
            true
          );
        });

        /*
         * TODO: When the licenses of the Trial clusters expire, add license expiration tests
         * (Trial licenses expire 2017-09-14)
         */
      });
    });

    describe('with standalone cluster', () => {
      const { setup, tearDown } = getLifecycleMethods(getService, getPageObjects);

      before(async () => {
        await setup('x-pack/test/functional/es_archives/monitoring/standalone_cluster', {
          from: 'Feb 4, 2019 @ 17:50:00.000',
          to: 'Feb 4, 2019 @ 17:52:00.000',
        });

        await clusterList.closeAlertsModal();
        await clusterList.assertDefaults();
      });

      after(async () => {
        await tearDown();
      });

      it('displays a standalone cluster', async () => {
        expect(await clusterList.hasCluster('__standalone_cluster__')).to.be(true);
        expect(await clusterList.hasCluster('lfhHkgqfTy2Vy3SvlPSvXg')).to.be(true);
      });
    });

    describe('with all basic license clusters', () => {
      const { setup, tearDown } = getLifecycleMethods(getService, getPageObjects);

      const UNSUPPORTED_CLUSTER_UUID = 'kH7C358oRzK6bmNzTeLEug';
      const SUPPORTED_CLUSTER_UUID = 'NDKg6VXAT6-TaGzEK2Zy7g';

      before(async () => {
        await setup('x-pack/test/functional/es_archives/monitoring/multi_basic', {
          from: 'Sep 7, 2017 @ 20:12:04.011',
          to: 'Sep 7, 2017 @ 20:18:55.733',
        });

        await clusterList.closeAlertsModal();

        await clusterList.assertDefaults();
      });

      after(async () => {
        await tearDown();
      });

      describe('cluster row content', function () {
        // TODO: https://github.com/elastic/stack-monitoring/issues/31
        this.tags(['skipCloud']);

        it('non-primary basic cluster shows NA for everything', async () => {
          expect(await clusterList.getClusterName(UNSUPPORTED_CLUSTER_UUID)).to.be('staging');
          expect(await clusterList.getClusterStatus(UNSUPPORTED_CLUSTER_UUID)).to.be('-');
          expect(await clusterList.getClusterNodesCount(UNSUPPORTED_CLUSTER_UUID)).to.be('-');
          expect(await clusterList.getClusterIndicesCount(UNSUPPORTED_CLUSTER_UUID)).to.be('-');
          expect(await clusterList.getClusterDataSize(UNSUPPORTED_CLUSTER_UUID)).to.be('-');
          expect(await clusterList.getClusterLogstashCount(UNSUPPORTED_CLUSTER_UUID)).to.be('-');
          expect(await clusterList.getClusterKibanaCount(UNSUPPORTED_CLUSTER_UUID)).to.be('-');
          expect(await clusterList.getClusterLicense(UNSUPPORTED_CLUSTER_UUID)).to.be(
            'Basic\nExpires 29 Aug 30'
          );
        });

        it('primary basic cluster shows cluster metrics', async () => {
          // PageObjects.common.sleep(10000)
          expect(await clusterList.getClusterName(SUPPORTED_CLUSTER_UUID)).to.be('production');
          expect(await clusterList.getClusterStatus(SUPPORTED_CLUSTER_UUID)).to.be('Clear');
          expect(await clusterList.getClusterNodesCount(SUPPORTED_CLUSTER_UUID)).to.be('2');
          expect(await clusterList.getClusterIndicesCount(SUPPORTED_CLUSTER_UUID)).to.be('4');
          expect(await clusterList.getClusterDataSize(SUPPORTED_CLUSTER_UUID)).to.be('1.6 MB');
          expect(await clusterList.getClusterLogstashCount(SUPPORTED_CLUSTER_UUID)).to.be('2');
          expect(await clusterList.getClusterKibanaCount(SUPPORTED_CLUSTER_UUID)).to.be('1');
          expect(await clusterList.getClusterLicense(SUPPORTED_CLUSTER_UUID)).to.be(
            'Basic\nExpires 29 Aug 30'
          );
        });
      });

      describe('cluster row actions', function () {
        // TODO: https://github.com/elastic/stack-monitoring/issues/31
        this.tags(['skipCloud']);

        it('clicking the non-primary basic cluster shows a toast message', async () => {
          const basicClusterLink = await clusterList.getClusterLink(UNSUPPORTED_CLUSTER_UUID);
          await basicClusterLink.click();
          expect(await testSubjects.exists('monitoringLicenseWarning', { timeout: 2000 })).to.be(
            true
          );
        });

        it('clicking the primary basic cluster goes to overview', async function () {
          const primaryBasicClusterLink = await clusterList.getClusterLink(SUPPORTED_CLUSTER_UUID);
          await primaryBasicClusterLink.click();

          expect(await clusterOverview.isOnClusterOverview()).to.be(true);
          expect(await clusterOverview.getClusterName()).to.be('production');
          await PageObjects.monitoring.closeAlertsModal();

          await PageObjects.monitoring.clickBreadcrumb('~breadcrumbClusters'); // reset for next test
        });
      });
    });

    describe('Alerts', () => {
      const { setup, tearDown } = getLifecycleMethods(getService, getPageObjects);

      before(async () => {
        await setup('x-pack/test/functional/es_archives/monitoring/multicluster', {
          from: 'Aug 15, 2017 @ 21:00:00.000',
          to: 'Aug 16, 2017 @ 00:00:00.000',
        });
      });

      after(async () => {
        await tearDown();

        await alertsService.deleteAlerts();

        await browser.clearLocalStorage();
      });

      it('should show a toast when alerts are created successfully', async () => {
        await clusterList.acceptAlertsModal();
        await clusterList.confirmWatcherMigrationDone();
        expect(await testSubjects.exists('alertsCreatedToast', { timeout: 10000 })).to.be(true);
      });
    });
  });
}
